@@ -24,10 +24,9 @@ proctable_post_config(server_rec * main_server, apr_pool_t * configpool)
 	fcgid_procnode *ptmpnode = NULL;
 	int i;
 	apr_status_t rv;
-	char tempname[L_tmpnam];
 
 	/* Create share memory */
-	if ((rv = apr_shm_create(&g_sharemem, shmem_size, tmpnam(tempname),
+	if ((rv = apr_shm_create(&g_sharemem, shmem_size, NULL,
 							 main_server->process->pconf)) != APR_SUCCESS)
 	{
 		ap_log_error(APLOG_MARK, APLOG_EMERG, rv, main_server,
