@@ -1,5 +1,4 @@
 #include "httpd.h"
-#include "ap_mpm.h"
 #include "http_request.h"
 #include "apr_strings.h"
 #include "apr_portable.h"
@@ -14,7 +13,7 @@
 #include "fcgid_protocol.h"
 #include "fcgid_bucket.h"
 #define FCGID_APPLY_TRY_COUNT 2
-#define FCGID_REQUEST_COUNT 3
+#define FCGID_REQUEST_COUNT 32
 
 static int g_variables_inited = 0;
 static int g_busy_timeout;
@@ -165,6 +164,7 @@ static int getsfunc_fcgid_BRIGADE(char *buf, int len, void *arg)
 	apr_status_t rv;
 	int done = 0;
 	int getLF = 0;
+	int getColon = 0;
 
 	while ((dst < dst_end) && !done && !APR_BUCKET_IS_EOS(e)) {
 		const char *bucket_data;
@@ -194,11 +194,16 @@ static int getsfunc_fcgid_BRIGADE(char *buf, int len, void *arg)
 		if (bucket_data_len == 0)
 			return 0;
 
+		/* Base on RFC2616 section 4.2 */
 		src = bucket_data;
 		src_end = bucket_data + bucket_data_len;
 		while ((src < src_end) && (dst < dst_end) && !done) {
-			if (getLF && *src != ' ' && *src != '\t') {
+			if (*src == ':')
+				getColon = 1;
+
+			if (getLF && ((*src != ' ' && *src != '\t') || !getColon)) {
 				done = 1;
+				getColon = 0;
 				break;
 			} else if (getLF && (*src == ' ' || *src == '\t')) {
 				*dst++ = '\r';
@@ -234,7 +239,7 @@ handle_request(request_rec * r, const char *argv0,
 	server_rec *main_server = r->server;
 	fcgid_command fcgi_request;
 	fcgid_bucket_ctx *bucket_ctx;
-	int i, stopping, cond_status;
+	int i, j, cond_status;
 	apr_status_t rv;
 	apr_bucket_brigade *brigade_stdout;
 	char sbuf[MAX_STRING_LEN];
@@ -250,8 +255,12 @@ handle_request(request_rec * r, const char *argv0,
 	}
 
 	bucket_ctx = apr_pcalloc(request_pool, sizeof(*bucket_ctx));
-	if (!bucket_ctx)
+	if (!bucket_ctx) {
+		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
+					 "mod_fcgid: apr_calloc of %d bytes failed in handle_request function",
+					 sizeof(*bucket_ctx));
 		return HTTP_INTERNAL_SERVER_ERROR;
+	}
 	bucket_ctx->ipc.connect_timeout = g_connect_timeout;
 	bucket_ctx->ipc.communation_timeout = g_comm_timeout;
 	bucket_ctx->ipc.request = r;
@@ -259,10 +268,9 @@ handle_request(request_rec * r, const char *argv0,
 							  bucket_ctx_cleanup, apr_pool_cleanup_null);
 
 	/* Try to get a connected ipc handle */
-	stopping = 0;
-	for (i = 0; i < FCGID_REQUEST_COUNT && !stopping; i++) {
+	for (i = 0; i < FCGID_REQUEST_COUNT; i++) {
 		/* Apply a free process slot, send a spawn request if I can't get one */
-		for (i = 0; i < FCGID_APPLY_TRY_COUNT && !stopping; i++) {
+		for (j = 0; j < FCGID_APPLY_TRY_COUNT; j++) {
 			int mpm_state = 0;
 			apr_ino_t inode =
 				wrapper_conf ? wrapper_conf->inode : r->finfo.inode;
@@ -276,6 +284,13 @@ handle_request(request_rec * r, const char *argv0,
 								   inode, shareid);
 
 			/* Apply a process slot */
+			bucket_ctx->procnode =
+				apply_free_procnode(r->server, &fcgi_request);
+			if (bucket_ctx->procnode)
+				break;
+
+			apr_sleep(apr_time_from_sec(1));
+
 			bucket_ctx->procnode =
 				apply_free_procnode(r->server, &fcgi_request);
 			if (bucket_ctx->procnode)
@@ -283,13 +298,6 @@ handle_request(request_rec * r, const char *argv0,
 
 			/* Send a spawn request if I can't get a process slot */
 			procmgr_post_spawn_cmd(&fcgi_request, r);
-
-			/* Is it stopping? */
-			if (ap_mpm_query(AP_MPMQ_MPM_STATE, &mpm_state) == APR_SUCCESS
-				&& mpm_state == AP_MPMQ_STOPPING) {
-				stopping = 1;
-				break;
-			}
 		}
 
 		/* Connect to the fastcgi server */
@@ -309,7 +317,7 @@ handle_request(request_rec * r, const char *argv0,
 
 	/* Now I get a connected ipc handle */
 	if (!bucket_ctx->procnode) {
-		ap_log_error(APLOG_MARK, APLOG_INFO, 0, r->server,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, 0, r->server,
 					 "mod_fcgid: can't apply process slot for %s", argv0);
 		return HTTP_SERVICE_UNAVAILABLE;
 	}
@@ -320,7 +328,7 @@ handle_request(request_rec * r, const char *argv0,
 	if ((rv =
 		 proc_write_ipc(main_server, &bucket_ctx->ipc,
 						output_brigade)) != APR_SUCCESS) {
-		ap_log_error(APLOG_MARK, APLOG_INFO, rv, r->server,
+		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
 					 "mod_fcgid: write data to fastcgi server error");
 		bucket_ctx->has_error = 1;
 		return HTTP_INTERNAL_SERVER_ERROR;
@@ -329,8 +337,11 @@ handle_request(request_rec * r, const char *argv0,
 	/* Create brigade */
 	brigade_stdout =
 		apr_brigade_create(request_pool, r->connection->bucket_alloc);
-	if (!brigade_stdout)
+	if (!brigade_stdout) {
+		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
+			"mod_fcgid: apr_brigade_create failed in handle_request function");
 		return HTTP_INTERNAL_SERVER_ERROR;
+	}
 	APR_BRIGADE_INSERT_TAIL(brigade_stdout,
 							ap_bucket_fcgid_header_create(r->connection->
 														  bucket_alloc,
@@ -371,6 +382,8 @@ handle_request(request_rec * r, const char *argv0,
 	if ((rv =
 		 ap_pass_brigade(r->output_filters,
 						 brigade_stdout)) != APR_SUCCESS) {
+		ap_log_error(APLOG_MARK, APLOG_WARNING, rv, r->server,
+					 "mod_fcgid: ap_pass_brigade failed in handle_request function");
 		return HTTP_INTERNAL_SERVER_ERROR;
 	}
 
@@ -431,7 +444,7 @@ int bridge_request(request_rec * r, const char *argv0,
 									AP_MODE_READBYTES,
 									APR_BLOCK_READ,
 									HUGE_STRING_LEN)) != APR_SUCCESS) {
-			ap_log_error(APLOG_MARK, APLOG_INFO, rv,
+			ap_log_error(APLOG_MARK, APLOG_WARNING, rv,
 						 main_server,
 						 "mod_fcgid: can't get data from http client");
 			apr_brigade_destroy(output_brigade);
@@ -474,7 +487,17 @@ int bridge_request(request_rec * r, const char *argv0,
 									   sizeof(*stdin_request_header),
 									   apr_bucket_free,
 									   r->connection->bucket_alloc);
-			apr_bucket_copy(bucket_input, &bucket_stdin);
+			if (APR_BUCKET_IS_HEAP(bucket_input))
+				apr_bucket_copy(bucket_input, &bucket_stdin);
+			else {
+				/* mod_ssl have a bug? */
+				char *pcopydata =
+					apr_bucket_alloc(len, r->connection->bucket_alloc);
+				memcpy(pcopydata, data, len);
+				bucket_stdin =
+					apr_bucket_heap_create(pcopydata, len, apr_bucket_free,
+										   r->connection->bucket_alloc);
+			}
 			if (!stdin_request_header || !bucket_header || !bucket_stdin
 				|| !init_header(FCGI_STDIN, 1, len, 0,
 								stdin_request_header)) {
