@@ -22,7 +22,8 @@ typedef struct {
 	int spawn_score;
 	int termination_score;
 	int max_process_count;
-	int default_max_class_process_count;
+	int max_class_process_count;
+	int min_class_process_count;
 	int output_buffersize;
 	apr_table_t *default_init_env;
 
@@ -32,6 +33,8 @@ typedef struct {
 	int ipc_comm_timeout_overwrite;
 	int busy_timeout;
 	int busy_timeout_overwrite;
+
+	int php_fix_pathinfo_enable;
 } fcgid_server_conf;
 
 typedef struct {
@@ -89,9 +92,13 @@ int get_spawnscore_uplimit(server_rec * s);
 const char *set_max_process(cmd_parms * cmd, void *dummy, const char *arg);
 int get_max_process(server_rec * s);
 
-const char *set_default_max_class_process(cmd_parms * cmd, void *dummy,
-										  const char *arg);
-int get_default_max_class_process(server_rec * s);
+const char *set_max_class_process(cmd_parms * cmd, void *dummy,
+								  const char *arg);
+int get_max_class_process(server_rec * s);
+
+const char *set_min_class_process(cmd_parms * cmd, void *dummy,
+								  const char *arg);
+int get_min_class_process(server_rec * s);
 
 const char *set_ipc_connect_timeout(cmd_parms * cmd, void *dummy,
 									const char *arg);
@@ -114,4 +121,8 @@ const char *set_wrapper_config(cmd_parms * cmd, void *dummy,
 							   const char *wrapper, const char *extension);
 fcgid_wrapper_conf *get_wrapper_info(const char *cgipath, request_rec * r);
 
+const char *set_php_fix_pathinfo_enable(cmd_parms * cmd, void *dummy,
+										const char *arg);
+int get_php_fix_pathinfo_enable(server_rec * s);
+
 #endif
